@@ -61,9 +61,15 @@ class DefaultRequirements(SuiteRequirements):
         return exclusions.only_on(["sqlite"])
 
     @property
-    def fk_names(self):
-        """foreign key constraints always have names in the DB"""
-        return exclusions.fails_on("sqlite")
+    def foreign_key_name_reflection(self):
+        """backend can reflect foreign key names"""
+
+        # issue here was fixed in SQLAlchemy #12954 for sqlite, 2.0
+        # release
+        return exclusions.skip_if(
+            lambda config: not sqla_compat.sqla_2_0_25
+            and exclusions.against(config, "sqlite")
+        )
 
     @property
     def reflects_fk_options(self):
@@ -374,7 +380,7 @@ class DefaultRequirements(SuiteRequirements):
         )
 
         version_high = exclusions.only_if(
-            lambda _: not compat.py313, "python 3.13 does not work right now"
+            lambda _: not compat.py314, "python 3.14 does not work right now"
         )
 
         sqlalchemy = exclusions.only_if(
