@@ -1,4 +1,5 @@
 """Test op functions against MSSQL."""
+
 from __future__ import annotations
 
 from typing import Any
@@ -6,8 +7,13 @@ from typing import Dict
 
 from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
+from sqlalchemy import Computed
+from sqlalchemy import DATE
+from sqlalchemy import DATETIME
 from sqlalchemy import exc
 from sqlalchemy import ForeignKey
+from sqlalchemy import func
+from sqlalchemy import Identity
 from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
@@ -17,10 +23,10 @@ from sqlalchemy import text
 
 from alembic import command
 from alembic import op
+from alembic import testing
 from alembic import util
 from alembic.testing import assert_raises_message
 from alembic.testing import combinations
-from alembic.testing import config
 from alembic.testing import eq_
 from alembic.testing import expect_warnings
 from alembic.testing import fixture
@@ -31,7 +37,6 @@ from alembic.testing.env import three_rev_fixture
 from alembic.testing.fixtures import capture_context_buffer
 from alembic.testing.fixtures import op_fixture
 from alembic.testing.fixtures import TestBase
-from alembic.util import sqla_compat
 
 
 class FullEnvironmentTests(TestBase):
@@ -118,9 +123,9 @@ class OpTest(TestBase):
             expected_nullability = not existing_nullability
             args["nullable"] = expected_nullability
         else:
-            args[
-                "existing_nullable"
-            ] = expected_nullability = existing_nullability
+            args["existing_nullable"] = expected_nullability = (
+                existing_nullability
+            )
 
         op.alter_column("t", "c", **args)
 
@@ -157,6 +162,12 @@ class OpTest(TestBase):
         context.assert_("ALTER TABLE tests ALTER COLUMN col BIT NOT NULL")
 
     def test_drop_index(self):
+        context = op_fixture("mssql")
+        op.drop_index("my_idx", table_name="my_table")
+        context.assert_contains("DROP INDEX my_idx ON my_table")
+
+    def test_drop_index_w_tablename_legacy(self):
+        """#1243"""
         context = op_fixture("mssql")
         op.drop_index("my_idx", "my_table")
         context.assert_contains("DROP INDEX my_idx ON my_table")
@@ -369,12 +380,11 @@ class OpTest(TestBase):
             "exec('alter table t drop constraint ' + @const_name)"
         )
 
-    @config.requirements.computed_columns_api
     def test_add_column_computed(self):
         context = op_fixture("mssql")
         op.add_column(
             "t1",
-            Column("some_column", Integer, sqla_compat.Computed("foo * 5")),
+            Column("some_column", Integer, Computed("foo * 5")),
         )
         context.assert_("ALTER TABLE t1 ADD some_column AS (foo * 5)")
 
@@ -456,14 +466,13 @@ class OpTest(TestBase):
         )
 
     @combinations(
-        (lambda: sqla_compat.Computed("foo * 5"), lambda: None),
-        (lambda: None, lambda: sqla_compat.Computed("foo * 5")),
+        (lambda: Computed("foo * 5"), lambda: None),
+        (lambda: None, lambda: Computed("foo * 5")),
         (
-            lambda: sqla_compat.Computed("foo * 42"),
-            lambda: sqla_compat.Computed("foo * 5"),
+            lambda: Computed("foo * 42"),
+            lambda: Computed("foo * 5"),
         ),
     )
-    @config.requirements.computed_columns
     def test_alter_column_computed_not_supported(self, sd, esd):
         op_fixture("mssql")
         assert_raises_message(
@@ -478,7 +487,6 @@ class OpTest(TestBase):
             existing_server_default=esd(),
         )
 
-    @config.requirements.identity_columns
     @combinations(
         ({},),
         (dict(always=True),),
@@ -489,7 +497,7 @@ class OpTest(TestBase):
         context = op_fixture("mssql")
         op.add_column(
             "t1",
-            Column("some_column", Integer, sqla_compat.Identity(**kw)),
+            Column("some_column", Integer, Identity(**kw)),
         )
         if "start" in kw or "increment" in kw:
             options = "(%s,%s)" % (
@@ -504,14 +512,13 @@ class OpTest(TestBase):
         )
 
     @combinations(
-        (lambda: sqla_compat.Identity(), lambda: None),
-        (lambda: None, lambda: sqla_compat.Identity()),
+        (lambda: Identity(), lambda: None),
+        (lambda: None, lambda: Identity()),
         (
-            lambda: sqla_compat.Identity(),
-            lambda: sqla_compat.Identity(),
+            lambda: Identity(),
+            lambda: Identity(),
         ),
     )
-    @config.requirements.identity_columns
     def test_alter_column_identity_add_not_supported(self, sd, esd):
         op_fixture("mssql")
         assert_raises_message(
@@ -573,3 +580,28 @@ class RoundTripTest(TestBase):
 
     # don't know if a default constraint can be explicitly named, but
     # the path is the same as the check constraint, so it should be good
+
+    @testing.variation("op", ["drop", "alter"])
+    def test_issue_1744(self, ops_context, connection, metadata, op):
+        access = Table(
+            "access",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("created_at", DATE, server_default=func.getdate()),
+        )
+        access.create(connection)
+
+        if op.alter:
+            ops_context.alter_column(
+                "access",
+                "created_at",
+                existing_type=DATETIME(),
+                type_=DATETIME(timezone=True),
+                server_default=func.getdate(),
+                existing_nullable=False,
+                existing_server_default=text("(getdate())"),
+            )
+        elif op.drop:
+            ops_context.drop_column(
+                "access", "created_at", mssql_drop_default=True
+            )
