@@ -128,11 +128,21 @@ class PostgresqlOpTest(TestBase):
         op.create_index("i", "t", ["c1", "c2"], unique=False)
         context.assert_("CREATE INDEX i ON t (c1, c2)")
 
-    def test_create_index_postgresql_if_not_exists(self):
+    def test_create_index_if_not_exists(self):
         context = op_fixture("postgresql")
         op.create_index("i", "t", ["c1", "c2"], if_not_exists=True)
         context.assert_("CREATE INDEX IF NOT EXISTS i ON t (c1, c2)")
 
+    def test_create_fk_postgresql_not_valid(self):
+        context = op_fixture("postgresql")
+        op.create_foreign_key(
+            "i", "t1", "t2", ["c1"], ["c2"], postgresql_not_valid=True
+        )
+        context.assert_(
+            "ALTER TABLE t1 ADD CONSTRAINT i FOREIGN KEY(c1) "
+            "REFERENCES t2 (c2) NOT VALID"
+        )
+
     @config.combinations("include_table", "no_table", argnames="include_table")
     def test_drop_index_postgresql_concurrently(self, include_table):
         context = op_fixture("postgresql")
@@ -146,7 +156,7 @@ class PostgresqlOpTest(TestBase):
             op.drop_index("geocoded", postgresql_concurrently=True)
         context.assert_("DROP INDEX CONCURRENTLY geocoded")
 
-    def test_drop_index_postgresql_if_exists(self):
+    def test_drop_index_if_exists(self):
         context = op_fixture("postgresql")
         op.drop_index("geocoded", if_exists=True)
         context.assert_("DROP INDEX IF EXISTS geocoded")
@@ -158,6 +168,16 @@ class PostgresqlOpTest(TestBase):
             "ALTER TABLE t ALTER COLUMN c TYPE INTEGER USING c::integer"
         )
 
+    def test_add_column_if_not_exists(self):
+        context = op_fixture("postgresql")
+        op.add_column("t", Column("c", Integer), if_not_exists=True)
+        context.assert_("ALTER TABLE t ADD COLUMN IF NOT EXISTS c INTEGER")
+
+    def test_drop_column_if_exists(self):
+        context = op_fixture("postgresql")
+        op.drop_column("t", "c", if_exists=True)
+        context.assert_("ALTER TABLE t DROP COLUMN IF EXISTS c")
+
     def test_col_w_pk_is_serial(self):
         context = op_fixture("postgresql")
         op.add_column("some_table", Column("q", Integer, primary_key=True))
@@ -327,12 +347,13 @@ class PostgresqlOpTest(TestBase):
         op.drop_table_comment("t2", existing_comment="t2 table", schema="foo")
         context.assert_("COMMENT ON TABLE foo.t2 IS NULL")
 
-    @config.requirements.computed_columns
     def test_add_column_computed(self):
         context = op_fixture("postgresql")
         op.add_column(
             "t1",
-            Column("some_column", Integer, Computed("foo * 5")),
+            Column(
+                "some_column", Integer, Computed("foo * 5", persisted=True)
+            ),
         )
         context.assert_(
             "ALTER TABLE t1 ADD COLUMN some_column "
@@ -347,7 +368,6 @@ class PostgresqlOpTest(TestBase):
             lambda: Computed("foo * 5"),
         ),
     )
-    @config.requirements.computed_columns
     def test_alter_column_computed_not_supported(self, sd, esd):
         op_fixture("postgresql")
         assert_raises_message(
@@ -362,7 +382,6 @@ class PostgresqlOpTest(TestBase):
             existing_server_default=esd(),
         )
 
-    @config.requirements.identity_columns
     @combinations(
         ({}, None),
         (dict(always=True), None),
@@ -384,7 +403,6 @@ class PostgresqlOpTest(TestBase):
             "INTEGER GENERATED %s AS IDENTITY%s" % (qualification, options)
         )
 
-    @config.requirements.identity_columns
     @combinations(
         ({}, None),
         (dict(always=True), None),
@@ -408,7 +426,6 @@ class PostgresqlOpTest(TestBase):
             "GENERATED %s AS IDENTITY%s" % (qualification, options)
         )
 
-    @config.requirements.identity_columns
     def test_remove_identity_from_column(self):
         context = op_fixture("postgresql")
         op.alter_column(
@@ -421,7 +438,6 @@ class PostgresqlOpTest(TestBase):
             "ALTER TABLE t1 ALTER COLUMN some_column DROP IDENTITY"
         )
 
-    @config.requirements.identity_columns
     @combinations(
         ({}, dict(always=True), "SET GENERATED ALWAYS"),
         (
