@@ -6,9 +6,14 @@ from unittest.mock import patch
 
 import sqlalchemy as sa
 from sqlalchemy import Column
+from sqlalchemy import DateTime
+from sqlalchemy import ForeignKey
 from sqlalchemy import inspect
+from sqlalchemy import Integer
 from sqlalchemy import MetaData
+from sqlalchemy import String
 from sqlalchemy import Table
+from sqlalchemy import UniqueConstraint
 
 from alembic import autogenerate
 from alembic import command
@@ -185,10 +190,10 @@ class ScriptNamingTest(TestBase):
             script._rev_path(
                 script.versions, "12345", "this is a message", create_date
             ),
-            os.path.abspath(
+            Path(
                 "%s/versions/12345_this_is_a_"
                 "message_2012_7_25_15_8_5.py" % _get_staging_directory()
-            ),
+            ).absolute(),
         )
 
     @testing.combinations(
@@ -219,7 +224,7 @@ class ScriptNamingTest(TestBase):
             script._rev_path(
                 script.versions, "12345", "this is a message", create_date
             ),
-            os.path.abspath(expected % _get_staging_directory()),
+            Path(expected % _get_staging_directory()).absolute(),
         )
 
     def _test_tz(self, timezone_arg, given, expected):
@@ -235,7 +240,13 @@ class ScriptNamingTest(TestBase):
         with mock.patch(
             "alembic.script.base.datetime",
             mock.Mock(
-                datetime=mock.Mock(utcnow=lambda: given, now=lambda: given),
+                datetime=mock.Mock(
+                    now=lambda tz=None: (
+                        given.replace(tzinfo=datetime.timezone.utc)
+                        if timezone_arg
+                        else given
+                    )
+                ),
                 timezone=datetime.timezone,
             ),
         ):
@@ -276,6 +287,19 @@ class ScriptNamingTest(TestBase):
             ),
         )
 
+    def test_generates_a_date(self):
+        """test #1643"""
+        script = ScriptDirectory(
+            _get_staging_directory(),
+            file_template="%(rev)s_%(slug)s_"
+            "%(year)s_%(month)s_"
+            "%(day)s_%(hour)s_"
+            "%(minute)s_%(second)s",
+            timezone="utc",
+        )
+        # generates a date with no warnings
+        eq_(script._generate_create_date(), mock.ANY)
+
     def test_default_tz(self):
         self._test_tz(
             None,
@@ -369,6 +393,16 @@ class RevisionCommandTest(TestBase):
             rev_id="no@atsigns",
         )
 
+        assert_raises_message(
+            util.CommandError,
+            r"Character\(s\) ':' not allowed in "
+            "revision identifier 'no:colons'",
+            command.revision,
+            self.cfg,
+            message="some message",
+            rev_id="no:colons",
+        )
+
         assert_raises_message(
             util.CommandError,
             r"Character\(s\) '-, @' not allowed in revision "
@@ -1159,6 +1193,102 @@ class RewriterTest(TestBase):
         eq_(diffs[1][0], "execute")
         eq_(diffs[1][1], "STATEMENT")
 
+    def test_reordering_example_wo_copy(self):
+        """test related to #1692 in that we identified the recipe
+        for rewriting column ordering was using column.copy()"""
+
+        m = MetaData()
+        t1 = Table(
+            "my_table",
+            m,
+            Column("data", String(50)),
+            Column("created_at", DateTime),
+            Column("id", Integer, primary_key=True),
+            Column("updated_at", DateTime),
+            UniqueConstraint("data", name="uq_data"),
+        )
+
+        t2 = Table(
+            "my_other_table",
+            m,
+            Column("data", String(50)),
+            Column("created_at", DateTime),
+            Column("id", Integer, primary_key=True),
+            Column("other_id", ForeignKey("my_table.id")),
+            Column("updated_at", DateTime),
+            UniqueConstraint("data", name="uq_data"),
+        )
+
+        writer = autogenerate.Rewriter()
+
+        @writer.rewrites(ops.CreateTableOp)
+        def order_columns(context, revision, op):
+
+            special_names = {
+                "id": -100,
+                "created_at": 1001,
+                "updated_at": 1002,
+            }
+
+            cols_by_key = [
+                (
+                    (
+                        special_names.get(col.key, index)
+                        if isinstance(col, Column)
+                        else 2000
+                    ),
+                    col,  # note no copy()
+                )
+                for index, col in enumerate(op.columns)
+            ]
+
+            columns = [
+                col
+                for idx, col in sorted(cols_by_key, key=lambda entry: entry[0])
+            ]
+            return ops.CreateTableOp(
+                op.table_name, columns, schema=op.schema, **op.kw
+            )
+
+        directives = [
+            ops.MigrationScript(
+                util.rev_id(),
+                ops.UpgradeOps(
+                    ops=[
+                        ops.CreateTableOp.from_table(t1),
+                        ops.CreateTableOp.from_table(t2),
+                    ]
+                ),
+                ops.DowngradeOps(ops=[]),
+            )
+        ]
+
+        ctx, rev = mock.Mock(), mock.Mock()
+        writer(ctx, rev, directives)
+        eq_(
+            autogenerate.render_python_code(directives[0].upgrade_ops),
+            "# ### commands auto generated by Alembic - please adjust! ###\n"
+            "    op.create_table('my_table',\n"
+            "    sa.Column('id', sa.Integer(), nullable=False),\n"
+            "    sa.Column('data', sa.String(length=50), nullable=True),\n"
+            "    sa.Column('created_at', sa.DateTime(), nullable=True),\n"
+            "    sa.Column('updated_at', sa.DateTime(), nullable=True),\n"
+            "    sa.PrimaryKeyConstraint('id'),\n"
+            "    sa.UniqueConstraint('data', name='uq_data')\n"
+            "    )\n"
+            "    op.create_table('my_other_table',\n"
+            "    sa.Column('id', sa.Integer(), nullable=False),\n"
+            "    sa.Column('data', sa.String(length=50), nullable=True),\n"
+            "    sa.Column('other_id', sa.Integer(), nullable=True),\n"
+            "    sa.Column('created_at', sa.DateTime(), nullable=True),\n"
+            "    sa.Column('updated_at', sa.DateTime(), nullable=True),\n"
+            "    sa.ForeignKeyConstraint(['other_id'], ['my_table.id'], ),\n"
+            "    sa.PrimaryKeyConstraint('id'),\n"
+            "    sa.UniqueConstraint('data', name='uq_data')\n"
+            "    )\n"
+            "    # ### end Alembic commands ###",
+        )
+
 
 class MultiDirRevisionCommandTest(TestBase):
     def setUp(self):
@@ -1181,7 +1311,8 @@ class MultiDirRevisionCommandTest(TestBase):
     def test_multiple_dir_no_bases_invalid_version_path(self):
         assert_raises_message(
             util.CommandError,
-            "Path foo/bar/ is not represented in current version locations",
+            r"Path foo[/\\]bar is not represented in current version "
+            "locations",
             command.revision,
             self.cfg,
             message="x",
@@ -1377,49 +1508,27 @@ class NormPathTest(TestBase):
 
         script = ScriptDirectory.from_config(config)
 
-        def normpath(path):
-            return path.replace(os.pathsep, ":NORM:")
+        eq_(
+            script._version_locations,
+            [Path(_get_staging_directory(), "scripts", "versions").absolute()],
+        )
 
-        normpath = mock.Mock(side_effect=normpath)
-
-        with mock.patch("os.path.normpath", normpath):
-            eq_(
-                script._version_locations,
-                (
-                    os.path.abspath(
-                        os.path.join(
-                            _get_staging_directory(), "scripts", "versions"
-                        )
-                    ).replace(os.pathsep, ":NORM:"),
-                ),
-            )
-
-            eq_(
-                script.versions,
-                os.path.abspath(
-                    os.path.join(
-                        _get_staging_directory(), "scripts", "versions"
-                    )
-                ).replace(os.pathsep, ":NORM:"),
-            )
+        eq_(
+            Path(script.versions),
+            Path(_get_staging_directory(), "scripts", "versions").absolute(),
+        )
 
     def test_script_location_multiple(self):
         config = _multi_dir_testing_config()
 
         script = ScriptDirectory.from_config(config)
 
-        def _normpath(path):
-            return path.replace(os.pathsep, ":NORM:")
-
-        normpath = mock.Mock(side_effect=_normpath)
-
-        with mock.patch("os.path.normpath", normpath):
-            sd = Path(_get_staging_directory()).as_posix()
-            eq_(
-                script._version_locations,
-                [
-                    _normpath(os.path.abspath(sd + "/model1/")),
-                    _normpath(os.path.abspath(sd + "/model2/")),
-                    _normpath(os.path.abspath(sd + "/model3/")),
-                ],
-            )
+        sd = Path(_get_staging_directory()).as_posix()
+        eq_(
+            script._version_locations,
+            [
+                Path(sd, "model1").absolute(),
+                Path(sd, "model2").absolute(),
+                Path(sd, "model3").absolute(),
+            ],
+        )
